'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var cors = _interopDefault(require('cors'));
var express = _interopDefault(require('express'));
var fileUpload = _interopDefault(require('express-fileupload'));
var functions = require('firebase-functions');
var glob = _interopDefault(require('glob'));
var path = require('path');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

(function (RequestType) {
  RequestType["GET"] = "GET";
  RequestType["POST"] = "POST";
  RequestType["PUT"] = "PUT";
  RequestType["DELETE"] = "DELETE";
  RequestType["PATCH"] = "PATCH";
})(exports.RequestType || (exports.RequestType = {}));

var Endpoint = function Endpoint(name, requestType, handler, options) {
  this.name = name;
  this.requestType = requestType;
  this.handler = handler;
  this.options = options;

  if (!handler) {
    throw new Error('Please provide a endpoint request handler.');
  }

  this.name = name;
  this.handler = handler;
  this.requestType = requestType;
  this.options = options;
};
var Get = /*#__PURE__*/function (_Endpoint) {
  _inheritsLoose(Get, _Endpoint);

  function Get(handler, options) {
    return _Endpoint.call(this, undefined, exports.RequestType.GET, handler, options) || this;
  }

  return Get;
}(Endpoint);
var Post = /*#__PURE__*/function (_Endpoint2) {
  _inheritsLoose(Post, _Endpoint2);

  function Post(handler, options) {
    return _Endpoint2.call(this, undefined, exports.RequestType.POST, handler, options) || this;
  }

  return Post;
}(Endpoint);
var Put = /*#__PURE__*/function (_Endpoint3) {
  _inheritsLoose(Put, _Endpoint3);

  function Put(handler, options) {
    return _Endpoint3.call(this, undefined, exports.RequestType.PUT, handler, options) || this;
  }

  return Put;
}(Endpoint);
var Delete = /*#__PURE__*/function (_Endpoint4) {
  _inheritsLoose(Delete, _Endpoint4);

  function Delete(handler, options) {
    return _Endpoint4.call(this, undefined, exports.RequestType.DELETE, handler, options) || this;
  }

  return Delete;
}(Endpoint);
var Patch = /*#__PURE__*/function (_Endpoint5) {
  _inheritsLoose(Patch, _Endpoint5);

  function Patch(handler, options) {
    return _Endpoint5.call(this, undefined, exports.RequestType.PATCH, handler, options) || this;
  }

  return Patch;
}(Endpoint);

var _console = console,
    log = _console.log;
var FunctionParser = /*#__PURE__*/function () {
  function FunctionParser(props) {
    var _options$enableCors, _options$groupByFolde, _options$buildReactiv, _options$buildEndpoin;

    var rootPath = props.rootPath,
        exports = props.exports,
        options = props.options,
        _props$verbose = props.verbose,
        verbose = _props$verbose === void 0 ? false : _props$verbose;

    if (!rootPath) {
      throw new Error('rootPath is required to find the functions.');
    }

    this.rootPath = rootPath;
    this.exports = exports;
    this.verbose = verbose;
    this.enableCors = (_options$enableCors = options == null ? void 0 : options.enableCors) != null ? _options$enableCors : false;
    var groupByFolder = (_options$groupByFolde = options == null ? void 0 : options.groupByFolder) != null ? _options$groupByFolde : true;
    var buildReactive = (_options$buildReactiv = options == null ? void 0 : options.buildReactive) != null ? _options$buildReactiv : true;
    var buildEndpoints = (_options$buildEndpoin = options == null ? void 0 : options.buildEndpoints) != null ? _options$buildEndpoin : true;

    if (buildReactive) {
      this.buildReactiveFunctions(groupByFolder);
    }

    if (buildEndpoints) {
      this.buildRestfulApi(groupByFolder);
    }
  }

  var _proto = FunctionParser.prototype;

  _proto.buildReactiveFunctions = function buildReactiveFunctions(groupByFolder) {
    var _this = this;

    if (this.verbose) log('Reactive Functions - Building...');
    var functionFiles = glob.sync(this.rootPath + "/**/*.function.js", {
      cwd: this.rootPath,
      ignore: './node_modules/**'
    });
    functionFiles.forEach(function (file) {
      var filePath = path.parse(file);
      var directories = filePath.dir.split('/');
      var groupName = groupByFolder ? directories[directories.length - 2] || '' : directories[directories.length - 1] || '';
      var functionName = filePath.name.replace('.function', '');

      if (!process.env.FUNCTION_NAME || process.env.FUNCTION_NAME === functionName) {
        if (!_this.exports[groupName]) _this.exports[groupName] = {};
        if (_this.verbose) log("Reactive Functions - Added " + groupName + "/" + functionName);
        _this.exports[groupName] = _extends({}, _this.exports[groupName], require(file));
      }
    });
    if (this.verbose) log('Reactive Functions - Built');
  };

  _proto.buildRestfulApi = function buildRestfulApi(groupByFolder) {
    var _this2 = this;

    if (this.verbose) log('Restful Endpoints - Building...');
    var apiFiles = glob.sync(this.rootPath + "/**/*.endpoint.js", {
      cwd: this.rootPath,
      ignore: './node_modules/**'
    });
    var app = express();
    var groupRouters = new Map();
    apiFiles.forEach(function (file) {
      var filePath = path.parse(file);
      var directories = filePath.dir.split('/');
      var groupName = groupByFolder ? directories[directories.length - 2] || '' : directories[directories.length - 1] || '';
      var router = groupRouters.get(groupName);

      if (!router) {
        router = express.Router();
        groupRouters.set(groupName, router);
      }

      try {
        _this2.buildEndpoint(file, groupName, router);
      } catch (e) {
        throw new Error("Restful Endpoints - Failed to add the endpoint defined in " + file + " to the " + groupName + " Api.");
      }

      app.use('/', router);
      _this2.exports[groupName] = _extends({}, _this2.exports[groupName], {
        api: functions.https.onRequest(app)
      });
    });
    if (this.verbose) log('Restful Endpoints - Built');
  };

  _proto.buildEndpoint = function buildEndpoint(file, groupName, router) {
    var _endpoint$options, _endpoint$options2, _endpoint$options$mid, _endpoint$options3, _endpoint$options$mid2, _endpoint$options4, _endpoint$options$mid3, _endpoint$options5, _endpoint$options$mid4, _endpoint$options6, _endpoint$options$mid5, _endpoint$options7;

    var filePath = path.parse(file);

    var endpoint = require(file)["default"];

    var name = endpoint.name || filePath.name.replace('.endpoint', '');
    var handler = endpoint.handler;

    if (this.enableCors) {
      router.use(cors());
    } else if ((_endpoint$options = endpoint.options) != null && _endpoint$options.enableCors) {
      if (this.verbose) log("Cors enabled for " + name);
      router.use(cors());
    }

    if ((_endpoint$options2 = endpoint.options) != null && _endpoint$options2.enableFileUpload) {
      if (this.verbose) log("File upload enabled for " + name);
      router.use(fileUpload());
    }

    switch (endpoint.requestType) {
      case exports.RequestType.GET:
        router.get("/" + name, (_endpoint$options$mid = (_endpoint$options3 = endpoint.options) == null ? void 0 : _endpoint$options3.middlewares) != null ? _endpoint$options$mid : [], handler);
        break;

      case exports.RequestType.POST:
        router.post("/" + name, (_endpoint$options$mid2 = (_endpoint$options4 = endpoint.options) == null ? void 0 : _endpoint$options4.middlewares) != null ? _endpoint$options$mid2 : [], handler);
        break;

      case exports.RequestType.PUT:
        router.put("/" + name, (_endpoint$options$mid3 = (_endpoint$options5 = endpoint.options) == null ? void 0 : _endpoint$options5.middlewares) != null ? _endpoint$options$mid3 : [], handler);
        break;

      case exports.RequestType.DELETE:
        router["delete"]("/" + name, (_endpoint$options$mid4 = (_endpoint$options6 = endpoint.options) == null ? void 0 : _endpoint$options6.middlewares) != null ? _endpoint$options$mid4 : [], handler);
        break;

      case exports.RequestType.PATCH:
        router.patch("/" + name, (_endpoint$options$mid5 = (_endpoint$options7 = endpoint.options) == null ? void 0 : _endpoint$options7.middlewares) != null ? _endpoint$options$mid5 : [], handler);
        break;

      default:
        throw new Error("A unsupported RequestType was defined for a Endpoint.\n\n          Please make sure that the Endpoint file exports a RequestType\n          using the constants in src/system/constants/requests.ts.\n\n          **This value is required to add the Endpoint to the API**");
    }

    if (this.verbose) log("Restful Endpoints - Added " + groupName + "/" + endpoint.requestType + ":" + name);
  };

  return FunctionParser;
}();

exports.Delete = Delete;
exports.Endpoint = Endpoint;
exports.FunctionParser = FunctionParser;
exports.Get = Get;
exports.Patch = Patch;
exports.Post = Post;
exports.Put = Put;
//# sourceMappingURL=firebase-backend.cjs.development.js.map
